from featurize_jupyterlab.core import optimizer, option
from featurize_jupyterlab import g
from torch.optim import SGD


@optimizer('PyTorch SGD', 'Simple wrap of the SGD optimizer of PyTorch')
@option('lr', help='Learning Rate', type='number', required=True, default='0.1')
def sgd(lr):
    return SGD(g.model.parameters(), float(lr))



from featurize_jupyterlab.core import dataset, option, metadata
from torchvision import datasets, transforms


@dataset('MNIST Train', 'This is a simple wrap for torchvision.datasets.MNIST(train=True)')
@option('fold', help='Absolute fold path to the dataset', required=True, default="~/.minetorch_dataset/torchvision_mnist")
@metadata('banner', 'https://upload.wikimedia.org/wikipedia/commons/2/27/MnistExamples.png')
def mnist_train(fold):
    return datasets.MNIST(fold, download=True, train=True, transform=transforms.ToTensor())


@dataset('MNIST Test', 'This is a simple wrap for torchvision.datasets.MNIST(Train=False)')
@option('fold', help='Absolute fold path to the dataset', required=True, default="~/.minetorch_dataset/torchvision_mnist")
@metadata('banner', 'https://upload.wikimedia.org/wikipedia/commons/2/27/MnistExamples.png')
def mnist(fold):
    return datasets.MNIST(fold, download=True, train=False)


@dataset('Wider Face', 'Wider Face Dataset')
@metadata('banner', 'http://shuoyang1213.me/WIDERFACE/support/intro.jpg')
def wider_face():
    return True

@dataset('Microsoft COCO', 'Microsoft COCO dataset')
@metadata('banner', 'http://cocodataset.org/images/coco-logo.png')
def coco():
    return True

@dataset('Image Net', 'Image Net Dataset')
@metadata('banner', 'https://miro.medium.com/max/750/1*IlzW43-NtJrwqtt5Xy3ISA.jpeg')
def image_net():
    return True

@dataset('Kaggle TGS Dataset', 'Kaggle TGS Dataset')
@metadata('banner', 'https://storage.googleapis.com/kaggle-competitions/kaggle/10151/logos/header.png')
def kaggle_tgs():
    return True

@dataset('Kaggle SSDD', 'Kaggle SSDD Dataset')
@metadata('banner', 'https://storage.googleapis.com/kaggle-competitions/kaggle/14241/logos/header.png')
def kaggle_ssdd():
    return True



from featurize_jupyterlab.core import model, option, metadata
from torchvision import models
from torch import nn
import importlib
import torch.nn.functional as F

@model('MNIST Demo Net', 'A very simple net for the MNIST demo, input should be 28x28 with channel')
def demo():
    class Net(nn.Module):
        def __init__(self):
            super(Net, self).__init__()
            self.conv1 = nn.Conv2d(1, 10, kernel_size=5)
            self.conv2 = nn.Conv2d(10, 20, kernel_size=5)
            self.conv2_drop = nn.Dropout2d()
            self.fc1 = nn.Linear(320, 50)
            self.fc2 = nn.Linear(50, 10)

        def forward(self, x):
            x = F.relu(F.max_pool2d(self.conv1(x), 2))
            x = F.relu(F.max_pool2d(self.conv2_drop(self.conv2(x)), 2))
            x = x.view(-1, 320)
            x = F.relu(self.fc1(x))
            x = F.dropout(x, training=self.training)
            x = self.fc2(x)
            return F.log_softmax(x, dim=1)
    return Net()



import setuptools

# with open("README.md", "r") as fh:
#     long_description = fh.read()
# 
# with open('requirements.txt') as f:
#     required = f.read().splitlines()
 

setuptools.setup(
    name='featurize-package',
    description='Official packages for featurize.',
    version='0.0.3',
    packages=setuptools.find_packages(),
    url="https://github.com/louis-she/featurize-package",
    author='louis',
    author_email='chenglu.she@gmail.com',
    keywords='pytorch minecraft',
    include_package_data=True,
    # install_requires=required,
    # entry_points={
    #     'console_scripts': ['minetorch=minetorch.cli:cli'],
    # }
)



from featurize_jupyterlab.core import dataflow, option
from featurize_jupyterlab.processors import Processor
import cv2

@dataflow('Image Resizer', 'Resize image to any shape')
@option('size', help='Square size of the image')
class ImageResizer(Processor):
    def __init__(self, size):
        self.size = size

    def __call__(self, data):
        return cv2.resize(data, self.size)


@dataflow('Horizontal Flip', 'Horizontal flip the image with a given probability')
@option('probability', help='Probability to flip the image', default=0.5)
class HorizontalFlip(Processor):
    def __init__(self, probability):
        self.probability = probability

    def __call__(self, column, size):
        # TODO: do flip
        pass



from featurize_jupyterlab.core import loss
import torch.nn.functional as F


@loss('PyTorch cross_entropy', 'Simple wrap of the cross_entropy of PyTorch')
def cross_entropy():
    def loss(trainer, data):
        image, target = data
        output = trainer.model(image)
        return F.nll_loss(output, target)
    return loss
